"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ECSYThreeEntity = void 0;

var _ecsy = require("ecsy");

var _Object3DComponent = require("./components/Object3DComponent.js");

var _Object3DTagComponents = require("./components/Object3DTagComponents.js");

class ECSYThreeEntity extends _ecsy._Entity {
  addObject3DComponent(obj, parentEntity) {
    obj.entity = this;
    this.addComponent(_Object3DComponent.Object3DComponent, {
      value: obj
    });

    if (obj.type === "Audio" && obj.panner !== undefined) {
      this.addComponent(_Object3DTagComponents.PositionalAudioTagComponent);
    } else if (obj.type === "Audio") {
      this.addComponent(_Object3DTagComponents.AudioTagComponent);
    } else if (obj.type === "AudioListener") {
      this.addComponent(_Object3DTagComponents.AudioListenerTagComponent);
    } else if (obj.isCamera) {
      this.addComponent(_Object3DTagComponents.CameraTagComponent);

      if (obj.isOrthographicCamera) {
        this.addComponent(_Object3DTagComponents.OrthographicCameraTagComponent);
      } else if (obj.isPerspectiveCamera) {
        this.addComponent(_Object3DTagComponents.PerspectiveCameraTagComponent);

        if (obj.isArrayCamera) {
          this.addComponent(_Object3DTagComponents.ArrayCameraTagComponent);
        }
      }
    } else if (obj.type === "CubeCamera") {
      this.addComponent(_Object3DTagComponents.CubeCameraTagComponent);
    } else if (obj.isImmediateRenderObject) {
      this.addComponent(_Object3DTagComponents.ImmediateRenderObjectTagComponent);
    } else if (obj.isLight) {
      this.addComponent(_Object3DTagComponents.LightTagComponent);

      if (obj.isAmbientLight) {
        this.addComponent(_Object3DTagComponents.AmbientLightTagComponent);
      } else if (obj.isDirectionalLight) {
        this.addComponent(_Object3DTagComponents.DirectionalLightTagComponent);
      } else if (obj.isHemisphereLight) {
        this.addComponent(_Object3DTagComponents.HemisphereLightTagComponent);
      } else if (obj.isPointLight) {
        this.addComponent(_Object3DTagComponents.PointLightTagComponent);
      } else if (obj.isRectAreaLight) {
        this.addComponent(_Object3DTagComponents.RectAreaLightTagComponent);
      } else if (obj.isSpotLight) {
        this.addComponent(_Object3DTagComponents.SpotLightTagComponent);
      } else if (obj.isLightProbe) {
        this.addComponent(_Object3DTagComponents.LightProbeTagComponent);

        if (obj.isAmbientLightProbe) {
          this.addComponent(_Object3DTagComponents.AmbientLightProbeTagComponent);
        } else if (obj.isHemisphereLightProbe) {
          this.addComponent(_Object3DTagComponents.HemisphereLightProbeTagComponent);
        }
      }
    } else if (obj.isBone) {
      this.addComponent(_Object3DTagComponents.BoneTagComponent);
    } else if (obj.isGroup) {
      this.addComponent(_Object3DTagComponents.GroupTagComponent);
    } else if (obj.isLOD) {
      this.addComponent(_Object3DTagComponents.LODTagComponent);
    } else if (obj.isMesh) {
      this.addComponent(_Object3DTagComponents.MeshTagComponent);

      if (obj.isInstancedMesh) {
        this.addComponent(_Object3DTagComponents.InstancedMeshTagComponent);
      } else if (obj.isSkinnedMesh) {
        this.addComponent(_Object3DTagComponents.SkinnedMeshTagComponent);
      }
    } else if (obj.isLine) {
      this.addComponent(_Object3DTagComponents.LineTagComponent);

      if (obj.isLineLoop) {
        this.addComponent(_Object3DTagComponents.LineLoopTagComponent);
      } else if (obj.isLineSegments) {
        this.addComponent(_Object3DTagComponents.LineSegmentsTagComponent);
      }
    } else if (obj.isPoints) {
      this.addComponent(_Object3DTagComponents.PointsTagComponent);
    } else if (obj.isSprite) {
      this.addComponent(_Object3DTagComponents.SpriteTagComponent);
    } else if (obj.isScene) {
      this.addComponent(_Object3DTagComponents.SceneTagComponent);
    }

    if (parentEntity && parentEntity.hasComponent(_Object3DComponent.Object3DComponent)) {
      parentEntity.getObject3D().add(obj);
    }

    return this;
  }

  removeObject3DComponent(unparent = true) {
    const obj = this.getComponent(_Object3DComponent.Object3DComponent, true).value;

    if (unparent) {
      // Using "true" as the entity could be removed somewhere else
      obj.parent && obj.parent.remove(obj);
    }

    this.removeComponent(_Object3DComponent.Object3DComponent);

    for (let i = this._ComponentTypes.length - 1; i >= 0; i--) {
      const Component = this._ComponentTypes[i];

      if (Component.isObject3DTagComponent) {
        this.removeComponent(Component);
      }
    }

    obj.entity = null;
  }

  removeAllComponents(forceImmediate) {
    if (this.hasComponent(_Object3DComponent.Object3DComponent)) {
      this.removeObject3DComponent();
    }

    return super.removeAllComponents(forceImmediate);
  }

  remove(forceImmediate) {
    if (this.hasComponent(_Object3DComponent.Object3DComponent)) {
      const obj = this.getObject3D();
      obj.traverse(o => {
        if (o.entity) {
          this._entityManager.removeEntity(o.entity, forceImmediate);
        }

        o.entity = null;
      });
      obj.parent && obj.parent.remove(obj);
    } else {
      this._entityManager.removeEntity(this, forceImmediate);
    }
  }

  getObject3D() {
    const component = this.getComponent(_Object3DComponent.Object3DComponent);
    return component && component.value;
  }

}

exports.ECSYThreeEntity = ECSYThreeEntity;