"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initialize = initialize;

var _world = require("./world.js");

var _WebGLRendererSystem = require("./systems/WebGLRendererSystem.js");

var _WebGLRendererComponent = require("./components/WebGLRendererComponent.js");

var _three = require("three");

function initialize(world, options = {}) {
  let {
    renderer,
    animationLoop
  } = options;

  if (!world) {
    world = new _world.ECSYThreeWorld();
  }

  world.registerComponent(_WebGLRendererComponent.WebGLRendererComponent).registerSystem(_WebGLRendererSystem.WebGLRendererSystem, {
    priority: 999
  });

  if (!renderer) {
    renderer = new _three.WebGLRenderer({
      antialias: true
    });
    document.body.appendChild(renderer.domElement);
  }

  if (!animationLoop) {
    const clock = new _three.Clock();

    animationLoop = () => {
      world.execute(clock.getDelta(), clock.elapsedTime);
    };
  }

  renderer.setAnimationLoop(animationLoop);
  const scene = new _three.Scene();
  const sceneEntity = world.createEntity().addObject3DComponent(scene);
  const camera = new _three.PerspectiveCamera(90, window.innerWidth / window.innerHeight, 0.1, 1000);
  const cameraEntity = world.createEntity().addObject3DComponent(camera, sceneEntity);
  const rendererEntity = world.createEntity().addComponent(_WebGLRendererComponent.WebGLRendererComponent, {
    scene: sceneEntity,
    camera: cameraEntity,
    renderer: renderer
  });
  return {
    world,
    camera,
    scene,
    renderer,
    sceneEntity,
    cameraEntity,
    rendererEntity
  };
}